"""
Module for authors repsone of Inspirehep

The moduel provided a data class with serveral get methods
to make it easy to get authors information from the response of 
a request to Inspire hep API for author record.

"""

import datetime
from dataclasses import dataclass
from typing import List
from pyinspirehep.utils import (
    convert_json_timestamp,
    convert_to_bool,
    convert_to_date,
)
from .data_models import SingleRecordResponse


@dataclass
class AuthorMetadata:
    """Author Metadata according to Inspirehep API.
    """
    project_membership: List[dict] = None
    positions: List[dict] = None
    advisors: List[dict] = None
    email_addresses: List[dict] = None
    ids: List[dict] = None
    name: dict = None
    stub: bool = None
    status: str = None
    schema: str = None
    deleted: bool = None
    control_number: int = None
    legacy_version: str = None
    arxiv_categories: List[str] = None
    legacy_creation_date: datetime.datetime = None

    @classmethod
    def from_dict(cls, metadata: dict = None):
        if metadata is None:
            return AuthorMetadata()
        legacy_creation_date = convert_to_date(
            metadata.get("legacy_creation_date", None),
        )
        return AuthorMetadata(
            project_membership=metadata.get("project_membership", None),
            positions=metadata.get("positions", None),
            advisors=metadata.get("advisors", None),
            email_addresses=metadata.get("email_addresses", None),
            ids=metadata.get("ids", None),
            name=metadata.get("name", None),
            stub=convert_to_bool(metadata.get("stub", None)),
            status=metadata.get("status", None),
            schema=metadata.get("$schema", None),
            deleted=convert_to_bool(metadata.get("deleted", None)),
            control_number=int(metadata.get("control_number", None)),
            legacy_version=metadata.get("legacy_version", None),
            arxiv_categories=metadata.get("arxiv_categories", None),
            legacy_creation_date=legacy_creation_date,
        )


@dataclass
class Author(SingleRecordResponse):
    """Class to contain Inspirehep API single record response for Author.

    Attribtes
    ---------
    id : str
        The unique identifier of the object.

    created : datetime.datetime
        The UTC timestamp when object was created.

    updated : datetime.datetime
        The UTC timestamp when object was las updated.

    links : dict
        List of related links to current object.

    metadata : AuthorMetadata
        Information about the Author which is an instance AuthorMetadata object.

    """
    id: str
    created: datetime.datetime = None
    updated: datetime.datetime = None
    links: dict = None
    metadata: AuthorMetadata = None

    @classmethod
    def from_response(cls, respone: dict):
        """

        Parameters
        ----------
        respone : dict
            

        Returns
        -------
        SingleRecordResponse

        """
        id = respone.get("id", None)
        created = convert_json_timestamp(
            respone.get("created", None),
        )
        updated = convert_json_timestamp(
            respone.get("updated", None),
        )
        links = respone.get("links", {})
        metadata = respone.get("metadata", {})
        metadata = AuthorMetadata.from_dict(metadata)
        return Author(
            id=id,
            created=created,
            updated=updated,
            links=links,
            metadata=metadata,
        )

    @property
    def name(self):
        """Returns then `name` dict of `self.metadata`

        """
        return self.metadata.name

    @property
    def positions(self):
        """Returns the `positions` list of `self.metadata`

        """
        return self.metadata.positions

    def get_name(self):
        """Returns full name of the author.
        
        """
        return self.name.get("value", None)

    def get_names_native(self):
        """Returns the native name of the author if exists.
        
        """
        return self.name.get("native_names", None)

    def get_name_preferred(self):
        """Returns the preferred name of the author if exists.
        
        """
        return self.name.get("preferred_name", None)

    def get_number_of_positions(self):
        """Returns number of all of the positions the author has had.

        """
        if self.positions:
            return len(self.positions)
        else:
            return 0

    @property
    def email_addresses(self):
        return self.metadata.email_addresses

    def get_email_addresses(self):
        """Returns list of all email address of the author.
        
        """
        if self.email_addresses:
            return [email_info.get("value", None) for email_info in self.email_addresses]
        else:
            return None

    def get_email_current(self):
        """Returns the current email address
        
        Warning:
        The function will return the first email address which has current: True.

        """
        for email in self.email_addresses:
            if email.get("current", None):
                return email.get("value", None)
        return None

    @property
    def ids(self):
        return self.metadata.ids

    def get_id_orcid(self):
        """Returns orcid id of the author if exists
        
        """
        for id_ in self.ids:
            if id_.get('schema', None).upper() == 'ORCID':
                return id_.get("value", None)
        return None

    def get_id_cern(self):
        """Returns CERN id of the author if exists
        
        """
        for id_ in self.ids:
            if id_.get('schema', None).upper() == 'CERN':
                return id_.get("value", None)
        return None

    def get_id_linkedin(self):
        """Returns Linkedin id of the author if exists
        
        """
        for id_ in self.ids:
            if id_.get('schema', None).upper() == 'LINKEDIN':
                return id_.get("value", None)
        return None

    def get_id_linkedin(self):
        """Returns Linkedin id of the author if exists
        
        """
        for id_ in self.ids:
            if id_.get('schema', None).upper() == 'LINKEDIN':
                return id_.get("value", None)
        return None

    def get_id_inspire_bai(self):
        """Returns INSPIRE BAI of the author if exists
        
        """
        for id_ in self.ids:
            if id_.get('schema', None).upper() == 'INSPIRE BAI':
                return id_.get("value", None)
        return None

    def get_id_inspire_id(self):
        """Returns INSPIRE ID of the author if exists
        
        """
        for id_ in self.ids:
            if id_.get('schema', None).upper() == 'INSPIRE ID':
                return id_.get("value", None)
        return None

    @property
    def arxiv_categories(self):
        """Returns the `arxiv_categories` list of `self.metadata`

        """
        return self.metadata.arxiv_categories

    def get_arxiv_categories(self):
        """Returns list of arxiv categories of the author

        """
        return self.arxiv_categories

    @property
    def control_number(self):
        """Returns the `control_number` of `self.metadata`
        
        """
        return self.metadata.control_number

    def get_id(self, as_int=False):
        """Returns the author id in INSPIRE HEP
        
        """
        return self.control_number if as_int else str(self.control_number)

    @property
    def project_membereship(self):
        """Returns the 'proejct_membership' of `self.metadata`
        
        """
        return self.metadata.project_membership

    def get_project_memberships(self, current=False):
        """Returns the project memberships of the authors.
        

        Parameters
        ----------
        current : bool
            If current is True only current project memberships will be 
            returned. Else, all project memeberships will be returned.
        
        
        Returns
        -------
        list
            list of string which shows the proejct memberships.
        
        """
        if not self.project_membereship:
            return None
        if current:
            memberships = [
                project["name"]
                for project in self.project_membereship
                if project['current'] == True
                ]

        else:
            memberships = [
                project["name"]
                for project in self.project_membereship
                ]
        return memberships if memberships else None

    def get_project_memberships_ids(self, current=False):
        """Returns the project memberships ids of the authors.
        

        Parameters
        ----------
        current : bool
            If current is True only current project memberships will be 
            returned. Else, all project memeberships will be returned.
        

        Returns
        -------
        list
            list of string which shows the proejct memberships.
        
        """
        if not self.project_membereship:
            return None
        if current:
            memberships = [
                project["record"]['$ref'].split("/")[-1]
                for project in self.project_membereship
                if project['current'] == True
                ]
        else:
            memberships = [
                project["record"]['$ref'].split("/")[-1]
                for project in self.project_membereship
                ]
        return memberships if memberships else None

    @property
    def advisors(self):
        """Returns the `advisors` of `self.metadata`
        
        """
        return self.metadata.advisors

    def get_advisors(self):
        """Returns the advisors of the authors
        
        """
        if self.advisors:
            return [advisor['name'] for advisor in self.advisors]
        else:
            return None

    def get_advisors_id(self):
        """Returns the inspire hep if of advisors of the author
        
        """
        if self.advisors:
            return [
                advisor['record']['$ref'].split("/")[-1]
                for advisor in self.advisors
                ]
        else:
            return None

    @property
    def positions(self):
        """Returns 'position' attribute of `self.metadata`

        """
        return self.metadata.positions

    def get_institutions(self):
        """Returns name of institutions of the author

        """
        if self.positions:
            return [
                position.get("institution", None)
                for position in self.positions
                ]
        else:
            return None

    def get_institutions_ids(self):
        """Returns ids of institutions of the author

        """
        if self.positions:
            return [
                position["record"]["$ref"].split("/")[-1]
                for position in self.positions
                ]
        else:
            return None

    def get_positions(self):
        """Returns dict of positions of the authors
        """
        return self.positions



"""
A module to get all information from Inspirehep API.
"""

import os
import json
from tabnanny import verbose
from pyinspirehep.client import Client


class LiteratureClone:
    """Class to clone Literature Information.

    Example:
    >>> import os
    >>> from pathlib import Path
    >>> from pyinspirehep.contrib.clone import LiteratureClone
    >>> directory = os.path.join(Path.home(), "Desktop", "literature")
    >>> cloner = LiteratureClone(directory)
    >>> cloner.clone(0,1000,1000)
    current collection size = 478
    current collection size = 976
    ----------------------------------------------->
    Saving json file number = 1000
    -----------------------------------------------<
    >>> assert os.path.isfile(os.path.join(directory,"1000.json"))
    """

    def __init__(
        self,
        directory=None,
        record_numbers=500,
        verbose=2,
        ) -> None:
        """
        Parameters
        ----------
        directory : str or path
            Determines the directory to save the json files during clone
        record_numbers : int
            (Default value 500)
            The number of records per page
        verbose : int
            (Default value 2)
            Determines the amount of information to be printed during clone.
            When it is 2 the maximum inofrmation will be printed.
        """
        if directory is None:
            raise ValueError("You must determine the directory name to save cloned data")
        self.collection = []
        self.client = Client()
        self.record_numbers = record_numbers
        self.verbose = verbose
        self.directory = directory
        if os.path.isdir(self.directory):
            pass
        else:
            self.directory = os.makedirs(self.directory)

    def clean(self) -> None:
        self.collection = []

    def save(self, filename: str) -> None:
        """Writes content to json file.

        Parameters
        ----------
        filename : str
            (Default value = None)

        Returns
        -------
        None

        """
        filename = os.path.join(self.directory, filename)
        with open(filename, 'w') as f:
            json.dump(self.collection, f)

    def _get_by_control_number(self, start) -> list:
        """Gets literatures list by using control number field.

        Parameters
        ----------
        start : int

        Returns
        -------
        List
            list of all literatures by control_number between `start` and
            `start` + `self.record_number`.
        """
        return self.client.search_literature(
            q=f'control_number:{start}->{start+self.record_numbers}',
            size=self.record_numbers,
            )['hits']['hits']

    def clone(
        self,
        min_control_number=0,
        max_control_number=2000000,
        batch_record_number=10000,
        ):
        """Clones all literature data in given interval.

        Parameters
        ----------
        min_control_number : int
            The minimum control number to start cloning from
        max_control_number : int
            The maximum control number to stop further cloning
        batch_record_number : int
            (Default 10000)
            When the control numbers modolue to `batch_record_number`
            is 0 it will be saved in hard disc and the `self.collection`
            will be cleared.
        
        """
        for start in range(
            min_control_number,
            max_control_number,
            self.record_numbers
            ):
            current_records = self._get_by_control_number(start)
            self.collection.extend(current_records)
            if self.verbose >= 2:
                print(f"current collection size = {len(self.collection)}")
            if (start + self.record_numbers) % batch_record_number == 0:
                if self.verbose >=1:
                    print("----------------------------------------------->")
                    print(f"Saving json file number = {start + self.record_numbers}")
                self.save(str(start + self.record_numbers)+".json")
                if self.verbose >= 1:
                    print("-----------------------------------------------<")
                self.clean()


if __name__ == '__main__':
    import doctest
    doctest.testmod(verbose=True)



"""
Module for literature repsone of Inspirehep

The moduel provided a data class with serveral get methods
to make it easy to get literature information from the response of 
a request to Inspire hep API for literature record and
resulf of literature search.

"""

import datetime
from dataclasses import dataclass
from datetime import date
from typing import List
from pyinspirehep.utils import (
    convert_json_timestamp,
    convert_to_bool,
    convert_to_date,
)
from .data_models import SingleRecordResponse


@dataclass
class LiteratureMetadata:
    control_number: str
    abstracts: List[dict] = None
    arxiv_eprints: List[dict] = None
    authors: List[dict] = None
    author_count: int = None
    earliest_date: date = None
    citation_count: int = 0
    citation_count_without_self_citations: int = 0
    citeable: bool = None
    copyright: List[dict] = None
    core: bool = None
    curated: bool = None
    documents: List[dict] = None
    document_type: List[str] = None
    dois: List[dict] = None
    facet_author_name: List[str] = None
    figures: List[dict] = None
    first_author: List[dict] = None
    imprints: List[dict] = None
    inspire_categories: List[dict] = None
    keywords: List[dict] = None
    legacy_version: str = None
    legacy_creation_date: date = None
    license: List[dict] = None
    number_of_pages: int = 0
    preprint_date: date = None
    primary_arxiv_category: List[str] = None
    public_notes: List[dict] = None
    publication_info: List[dict] = None
    referenced_authors_bais: List[str] = None
    references: List[dict] = None
    refereed: bool = None
    schema: str = None
    texkeys: List[str] = None
    titles: List[dict] = None

    @classmethod
    def from_dict(cls, metadata: dict = None):
        if metadata is None:
            return LiteratureMetadata()
        legacy_creation_date = convert_to_date(
            metadata.get("legacy_creation_date", None),
        )
        earliest_date = convert_to_date(
            metadata.get("earliest_date", None)
        )
        preprint_date = convert_to_date(
            metadata.get("preprint_date", None)
        )
        return LiteratureMetadata(
            control_number=metadata["control_number"],
            abstracts=metadata.get("abstracts", None),
            arxiv_eprints=metadata.get("arxiv_eprints", None),
            authors=metadata.get("authors", None),
            author_count=metadata.get("author_count", None),
            earliest_date=earliest_date,
            citation_count=int(metadata.get("citation_count", 0)),
            citation_count_without_self_citations=metadata.get(
                "citation_count_without_self_citations", 0),
            citeable=convert_to_bool(metadata.get("citeable", None)),
            copyright=metadata.get("copyright", None),
            core=convert_to_bool(metadata.get("core", None)),
            curated=convert_to_bool(metadata.get("curated", None)),
            documents=metadata.get("documents", None),
            document_type=metadata.get("document_type", None),
            dois=metadata.get("dois", None),
            facet_author_name=metadata.get("facet_author_name", None),
            figures=metadata.get("figures", None),
            first_author=metadata.get("first_author", None),
            imprints=metadata.get("imprints", None),
            inspire_categories=metadata.get("inspire_categories", None),
            keywords=metadata.get("keywords", None),
            legacy_version=metadata.get("legacy_version", None),
            legacy_creation_date=legacy_creation_date,
            license=metadata.get("license", None),
            number_of_pages=metadata.get("number_of_pages", 0),
            preprint_date=preprint_date,
            primary_arxiv_category=metadata.get("primary_arxiv_category", 0),
            public_notes=metadata.get("public_notes", 0),
            publication_info=metadata.get("publication_info", 0),
            referenced_authors_bais=metadata.get("referenced_authors_bais", 0),
            references=metadata.get("references", 0),
            refereed=convert_to_bool(metadata.get("refereed", None)),
            schema=metadata.get("$schema", None),
            texkeys=metadata.get("texkeys", 0),
            titles=metadata.get("titles", 0),
        )


@dataclass
class Literature(SingleRecordResponse):
    """Class to contain Inspirehep API single record response for Literature.

    Attribtes
    ---------
    id : str
        The unique identifier of the object.

    created : datetime.datetime
        The UTC timestamp when object was created.

    updated : datetime.datetime
        The UTC timestamp when object was las updated.

    links : dict
        List of related links to current object.

    metadata : LiteratureMetadata
        Information about the Literature which is an instance LiteratureMetadata object.

    """
    id: str
    created: datetime.datetime = None
    updated: datetime.datetime = None
    links: dict = None
    metadata: LiteratureMetadata = None

    @classmethod
    def from_response(cls, respone: dict):
        """

        Parameters
        ----------
        respone : dict
            

        Returns
        -------
        SingleRecordResponse

        """
        id = respone.get("id", None)
        created = convert_json_timestamp(
            respone.get("created", None),
        )
        updated = convert_json_timestamp(
            respone.get("updated", None),
        )
        links = respone.get("links", {})
        metadata = respone.get("metadata", {})
        metadata = LiteratureMetadata.from_dict(metadata)
        return Literature(
            id=id,
            created=created,
            updated=updated,
            links=links,
            metadata=metadata,
        )

    def get_citation_count(self):
        return self.metadata.citation_count

    def get_references_ids(self):
        ids = []
        if self.metadata.references:
            for item in self.metadata.references:
                ids.append(item['record']['$ref'].split("/")[-1])
        return ids




import datetime
from typing import Iterable

def _convert_json_timestamp(
    timestamp_str: str,
    format: str = '%Y-%m-%dT%H:%M:%S%z',
    ) -> datetime.datetime:
    """Converts json timestamp to Pythons datetime object.

    Parameters
    ----------
    timestamp_str : str
        The timestamp string of json type
        
    format : str
         (Default value = '%Y-%m-%dT%H:%M:%S%z')
         The format to match the `timestamp_str` in order to create Python
         datetime object. 

    Returns
    -------
    datetime.datetime

    """
    if timestamp_str is not None:
        return datetime.datetime.strptime(
            timestamp_str,
            format,
        )
    else:
        return None

def convert_json_timestamp(
    timestamp_str: str,
    formats: Iterable[str] = None,
    ) -> datetime.datetime:
    """Creates Python datetime object form json timestamp.

    The function will try each format in `formats` in order to match the
    given `timestamp_str`. If a fomrat matches the datetime object wil be
    returned. If no format matches with the given `timestamp_str`, then
    ValueError will be raised.

    Parameters
    ----------
    timestamp_str : str
        The timestamp string of json type
        
    formats : Iterable[str]
         (Default value = None)
         An iterable of string which are the formats that will be tried.

    Returns
    -------
    datetime.datetime

    Raises
    -------
    ValueError
        When no format was matches this error will be raised.

    """
    if timestamp_str is None:
        return None
    if formats is None:
        formats = [
            '%Y-%m-%dT%H:%M:%S%z',
            '%Y-%m-%dT%H:%M:%S.%f%z',
            '%Y-%m-%dT%H:%M:%S%f%z',
        ]
    for format in formats:
        try:
            return _convert_json_timestamp(
                timestamp_str,
                format,
            )
        except ValueError:
            continue
    raise ValueError(
        f"time data '{timestamp_str}' does not "
        f"match fomrat {formats}"
    )


def convert_to_date(date_str: str, formats: list = None):
    if date_str is None:
        return None
    if formats is None:
        formats = [
            '%Y-%m-%d',
            ]
    for format in formats:
        try:
            return _convert_json_timestamp(
                date_str,
                format,
                ).date()
        except ValueError:
            continue
    raise ValueError(
        f"date data '{date_str}' does not "
        f"match fomrat in {formats}"
    )


def convert_to_bool(variable: str = None):
    """converts variable to boolean.

    It retruns None if the variable is not True or False or true or false.

    Parameters
    ----------
    variable : str

    Returns
    -------
    bool

    """
    if bool(variable) is not None:
        return bool(variable)
    else:
        return None


"""Client to use Inspirehep API.

The client.py moducle contains the base class `Client` which can be used to
get data from Inspirehep API.

"""

import requests
import time
from pyinspirehep.exception import (
    InspirehepPIDDoesNotExistError,
    InspirehepTooManyRequestsError,
)
from pyinspirehep.data_models import (
    SingleRecordResponse,
)
from pyinspirehep.author import Author
from pyinspirehep.literature import Literature


class Client:
    """Client to use Inspirehep API.
    """

    REST_API_URL = 'https://inspirehep.net/api/'

    SEARCH_RESULT_KEY = 'hits'

    IDENTIFIER_TYPES = [
        'literature',
        'authors',
        'institutions',
        'conferences',
        'seminars',
        'journals',
        'jobs',
        'experiments',
        'data',
        ]

    EXTERNAL_IDENTIFIER_TYPES = [
        'doi',
        'arxiv',
        'orcid',
        ]

    LIMIT_TIME = 5

    PAGINATION_LIMIT = 10000

    MAX_RECORDS_PER_PAGE = 1000

    MAX_PAGES = PAGINATION_LIMIT // MAX_RECORDS_PER_PAGE
    

    def __init__(self) -> None:
        self.session = self._init_session()

    def _init_session(self) -> requests.session:
        """Initialize session.
        """

        session = requests.session()
        headers = {
            'Accept': 'application/json',
            'User-Agent': 'pyinspirehep',
            'Content-Type': 'application/json',
            }
        session.headers.update(headers)
        return session

    def wait_429(self) -> None:
        """Wait before sending next request.

        Use to wait `LIMIT_TIME` seconds before sending new requests.
        """
        time.sleep(self.LIMIT_TIME)

    def _get(self, *args, **kwargs) -> dict:
        """Sends a GET request and returns json data.

        This method uses `requests.get` method to get data from API and
        and returns data as json.

        Parameters
        ----------
        *args :
            Passed to `requests.get` as *args.
            
        **kwargs :
            Passed to `requests.get` as **kwargs.

        Returns
        -------
        dict
            Result of json data will be returned as Python dict.

        Raises
        ------
        InspirehepPIDDoesNotExistError
            When the requested object was not found.

        InspirehepTooManyRequestsError
            When because of too many request the IP is blocked for
            a few seconds.

        """
        try:
            response = requests.get(*args, **kwargs)
        except requests.exceptions.ConnectionError as e:
            raise InspirehepTooManyRequestsError(str(e))
        data = response.json()
        if response.status_code == 200:
            return response.json()
        elif response.status_code == 404:
            raise InspirehepPIDDoesNotExistError(
                data.get('message', '404 status code'),
                )
        elif response.status_code == 429:
            raise InspirehepTooManyRequestsError(
                data.get('message', '429 status code'),
                )

    def _get_record(
        self,
        *args,
        identifier_type: str,
        identifier_value: str,
        ) -> dict:
        """

        Parameters
        ----------
        identifier_type : str
            The `identifier-value` type is a string which determines
            what type of object to be get. For example, it can be
            'authors', 'literature', etc.

        identifier_value : str
            The `identifier-value` is a number identifying the given record
            in the INSPIRE database (also called record ID or recid)

        *args :
            The fields that must be included in metadata. If not specified, 
            all possible fields in metadata will be send in response.


        Returns
        -------
        dict
            The json data part of the respone as Python dict will be returned.

        """
        args = [self.REST_API_URL, identifier_type, identifier_value]
        fields = ""
        if args:
            fields = ",".join(args)
        if fields:
            return self._get(
                Client._create_uri(*args),
                params={'fields':fields}
                )
        else:
            return self._get(
                Client._create_uri(*args)
                )

    def _create_q(
        metadata_field: str,
        nested_key: str = None,
        value: str = "",
        ) -> str:
        """Creates query to use in URL for a field

        Parameters
        ----------
        metadata_field : str
            
        nested_key : str
             (Default value = None)
        value : str
             (Default value = "")

        Returns
        -------
        str

        """
        if nested_key is None:
            return f"{metadata_field}:{value}"
        else:
            return f"{metadata_field}.{nested_key}:{value}"

    def _search(
        self,
        *args,
        identifier_type: str,
        **kwargs,
        ) -> dict:
        """

        Parameters
        ----------
        *args :
            The fields that must be included in metadata. If not specified, 
            all possible fields in metadata will be send in response. All
            items in the *args must be string. Using `Client._create_params`
            all items in *args will be concatenated and will be the value for
            fields key in query parameters in request URL.
            
        identifier_type : str
            
        **kwargs :
            Keyworod arguments will be passed to `Client._create_params` and 
            they will created values for query parameters such as size, page,
            sort and q.
            

        Returns
        -------

        """
        uri_args = [self.REST_API_URL, identifier_type]
        return self._get(
            Client._create_uri(*uri_args),
            params=Client._create_params(*args, **kwargs),
            )

    def _get_record_object(
        self,
        *args,
        identifier_type: str,
        identifier_value: str,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        *args:
            The fields that must be included in metadata.

        identifier_type : str
            
        identifier_value : str
            

        Returns
        -------
        SingleRecrodResponse

        """
        return SingleRecordResponse.from_response(
            self._get_record(
                *args,
                identifier_type=identifier_type,
                identifier_value=identifier_value,
                ),
            )

    @staticmethod
    def _create_params(
        *args,
        sorting: str = None,
        page: str = None,
        size: str = None,
        q: str = None,
        ) -> dict:
        """

        Parameters
        ----------
        *args:
            All items in *args must be string. The positional arguments in
            args will be used as filters of fields of metadata. If no
            positional arugemnt was provided, then all fields of metadata
            will be returned in result of the response.

        sorting : str
            (Default value = None). Determines the sort roder of objects
            in response.

        page : str
            (Default value = None). The page number to get in query.

        size : str
            (Default value = None). The number of records in page.

        q : str
            (Default value = None). The search query.

        Returns
        -------
        dict

        >>> client = Client()
        >>> client._create_params('name.value', 'positions', sorting='bestmatch', size=1000, page=1)
        {'sort': 'bestmatch', 'page': 1, 'size': 1000, 'fields': 'name.value,positions'}
        >>> client._create_params('name.value', 'positions', sorting='bestmatch', size=1000, page=1, q='control_number:1966745')
        {'sort': 'bestmatch', 'page': 1, 'size': 1000, 'q': 'control_number:1966745', 'fields': 'name.value,positions'}
        """
        params = {}
        if sorting is not None:
            params['sort'] = sorting  # The sort order
        if page is not None:
            params['page'] = page  # The page number
        if size is not None:
            params['size'] = size  # The number of results returned per page
        if q is not None:
            params['q'] = q  # The search query
        fields = None  # The fields in the metadata to be returned
        if args:
            fields = ",".join(args)
        if fields:
            params['fields'] = fields

        return params

    @staticmethod
    def _create_uri(*args, end: str = "") -> str:
        """

        Parameters
        ----------
        *args :
            
        end : str
            (Default value = "")

        Returns
        -------
        str

        """
        assert end in ("", "/")
        return "/".join(args) + end

    def get_literature(
        self,
        literature_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        literature_id : str

        *args :
            Items to include in metadata.
            

        Returns
        -------
        dict


        >>> client = Client()
        >>> paper = client.get_literature("451647")
        >>> paper["metadata"]["titles"][0]["title"]
        'The Large N limit of superconformal field theories and supergravity'
        """
        return self._get_record(
            *args,
            identifier_type='literature',
            identifier_value=literature_id,
            )

    def get_literature_citations(
        self,
        literature_id: str,
        size=1000,
        page=1,
        ):
        """

        Parameters
        ----------
        literature_id : str
            The id of literature that its citations are wanted

        Returns
        -------
        dict

        >>> client = Client()
        >>> paper = client.get_literature("1785369")
        >>> len(client.get_literature_citations("1785369")["hits"]["hits"][1]["metadata"]["references"])
        133
        """
        args=[
            self.REST_API_URL + 'literature',
            f"?q=refersto%3Arecid%3A{literature_id}" + f'&size={size}' + f'&page={page}',
            ]
        print(Client._create_uri(*args))
        return self._get(Client._create_uri(*args))

    def get_literature_object(
        self,
        literature_id: str,
        *args,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        literature_id : str

        *args :
            Items to include in metadata.            

        Returns
        -------
        SingleRecordResponse

        """
        return Literature.from_response(
            self.get_literature(literature_id, *args),
        )

    def search_literature(
        self,
        *args,
        sorting='mostrecent',
        size=1,
        page=1,
        q=None,
        ) -> dict:
        """

        Parameters
        ----------
        sorting : str
             (Default value = 'mostrecent')

        size : int
             (Default value = 1)

        page : int
             (Default value = 1)

        q :
             (Default value = None)

        name :
             (Default value = None)

        Returns
        -------
        dict

        >>> c = Client()
        >>> literature = c.search_literature(q="control_number:1->10000")

        """
        return self._search(
            *args,
            identifier_type='literature',
            sorting=sorting,
            size=size,
            page=page,
            q=q,
        )

    def get_author(
        self,
        author_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        author_id : str

        *args :
            Items to include in metadata.

        Returns
        -------
        dict

        """
        return self._get_record(
            *args,
            identifier_type='authors',
            identifier_value=author_id,
            )

    def get_author_object(
        self,
        author_id: str,
        *args,
        ) -> Author:
        """

        Parameters
        ----------
        author_id : str

        *args :
            Items to include in metadata.
            

        Returns
        -------
        Author

        """
        return Author.from_response(
            self.get_author(author_id, *args),
        )

    def search_authors(
        self,
        *args,
        sorting='bestmatch',
        size=1000,
        page=1,
        q=None,
        name=None,
        ) -> dict:
        """

        Parameters
        ----------
        sorting :
             (Default value = 'bestmatch')
        size :
             (Default value = 1)
        page :
             (Default value = 1)
        fields:
            (Default value = None)
        q :
             (Default value = None)
        name :
             (Default value = None)

        Returns
        -------
        dict

        """
        if name is not None:
            q = Client._create_q('name', 'value', name)
        return self._search(
            *args,
            identifier_type='authors',
            sorting=sorting,
            size=size,
            page=page,
            q=q,
            )

    def get_institution(
        self,
        institution_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        institution_id : str
            

        Returns
        -------
        dict

        """
        return self._get_record(
            *args,
            identifier_type='institutions',
            identifier_value=institution_id,
            )

    def get_institution_object(
        self,
        institution_id: str,
        *args,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        institution_id : str
            

        Returns
        -------
        SingleRecordResponse

        """
        return self._get_record_object(
            *args,
            identifier_type='institutions',
            identifier_value=institution_id,
            )

    def get_conference(
        self,
        conference_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        conference_id : str
            

        Returns
        -------
        dict

        """
        return self._get_record(
            *args,
            identifier_type='conferences',
            identifier_value=conference_id,
            )

    def get_conference_object(
        self,
        conference_id: str,
        *args,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        conference_id : str
            

        Returns
        -------
        SingleRecordResponse

        """
        return self._get_record_object(
            *args,
            identifier_type='conferences',
            identifier_value=conference_id,
        )

    def get_seminar(
        self,
        seminar_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        seminar_id : str
            

        Returns
        -------
        dict

        """
        return self._get_record(
            *args,
            identifier_type='seminars',
            identifier_value=seminar_id,
            )

    def get_seminar_object(
        self,
        seminar_id: str,
        *args,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        seminar_id : str
            

        Returns
        -------
        SingleRecordResponse

        """
        return self._get_record_object(
            *args,
            identifier_type='seminars',
            identifier_value=seminar_id,
        )

    def get_journal(
        self,
        journal_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        journal_id : str
            

        Returns
        -------
        dict

        """
        return self._get_record(
            *args,
            identifier_type='journals',
            identifier_value=journal_id,
            )

    def get_journal_object(
        self,
        journal_id: str,
        *args,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        journal_id : str
            

        Returns
        -------
        SingleRecordResponse

        """
        return self._get_record_object(
            *args,
            identifier_type='journals',
            identifier_value=journal_id,
        )

    def get_job(
        self,
        job_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        job_id : str
            

        Returns
        -------
        dict

        """
        return self._get_record(
            *args,
            identifier_type='jobs',
            identifier_value=job_id,
            )

    def get_job_object(
        self,
        job_id: str,
        *args,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        job_id : str
            

        Returns
        -------
        SingleRecordResponse

        """
        return self._get_record_object(
            *args,
            identifier_type='jobs',
            identifier_value=job_id,
        )

    def get_experiment(
        self,
        experiment_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        experiment_id : str
            

        Returns
        -------
        dict

        """
        return self._get_record(
            *args,
            identifier_type='experiments',
            identifier_value=experiment_id,
            )

    def get_experiment_object(
        self,
        experiment_id: str,
        *args,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        experiment_id : str
            

        Returns
        -------
        SingleRecordResponse

        """
        return self._get_record_object(
            *args,
            identifier_type='experiments',
            identifier_value=experiment_id,
        )

    def get_data(
        self,
        data_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        data_id : str
            

        Returns
        -------
        dict

        """
        return self._get_record(
            *args,
            identifier_type='data',
            identifier_value=data_id,
            )

    def get_data_object(
        self,
        data_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        data_id : str
            

        Returns
        -------
        SingleRecordResponse

        """
        return self._get_record_object(
            *args,
            identifier_type='data',
            identifier_value=data_id,
        )

    def get_doi(
        self,
        doi_identifier: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        doi_identifier : str
            

        Returns
        -------
        dict

        >>> client = Client()
        >>> literature_record = client.get_doi("10.1103/PhysRevLett.19.1264")
        >>> literature_record["metadata"]["titles"][-1]["title"]
        'A Model of Leptons'
        """
        return self._get_record(
            *args,
            identifier_type='doi',
            identifier_value=doi_identifier,
            )

    def get_doi_object(
        self,
        doi_identifier: str,
        *args,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        doi_identifier : str
            

        Returns
        -------
        SingleRecordResponse

        """
        return self._get_record_object(
            *args,
            identifier_type='doi',
            identifier_value=doi_identifier,
        )

    def get_arxiv(
        self,
        arxiv_identifier: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        arxiv_identifier : str
            

        Returns
        -------
        dict

        >>> client = Client()
        >>> literature_record = client.get_arxiv("1207.7214")
        >>> literature_record["metadata"]['titles'][-1]['title']
        'Observation of a new particle in the search for the Standard Model Higgs boson with the ATLAS detector at the LHC'
        >>> literature_record["metadata"]['titles'][-1]['source']
        'arXiv'
        >>> literature_record = client.get_arxiv("hep-ph/0603175")
        >>> literature_record["metadata"]['titles'][-1]['source']
        'arXiv'
        """
        return self._get_record(
            *args,
            identifier_type='arxiv',
            identifier_value=arxiv_identifier,
            )

    def get_arxiv_object(
        self,
        arxiv_identifier: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        arxiv_identifier : str
            

        Returns
        -------
        SingleRecordResponse
        
        """
        return self._get_record_object(
            *args,
            identifier_type='arxiv',
            identifier_value=arxiv_identifier,
            )

    def get_orcid(
        self,
        orcid_id: str,
        *args,
        ) -> dict:
        """

        Parameters
        ----------
        orcid_id : str
            

        Returns
        -------
        dict


        >>> client = Client()
        >>> author_record = client.get_orcid("0000-0003-3897-046X")
        >>> author_record["metadata"]["name"]["value"]
        'Seiberg, Nathan'
        """
        return self._get_record(
            *args,
            identifier_type='orcid',
            identifier_value=orcid_id,
            )

    def get_orcid_object(
        self,
        orcid_id: str,
        *args,
        ) -> SingleRecordResponse:
        """

        Parameters
        ----------
        orcid_id : str
            

        Returns
        -------
        SingleRecordResponse

        """
        return self._get_record_object(
            *args,
            identifier_type='orcid',
            identifier_value=orcid_id,
        )


if __name__ == '__main__':
    import doctest
    doctest.testmod(verbose=True)



# Import required functions
from setuptools import setup, find_packages

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

# Call setup function
setup(
    author="Javad Ebadi, Vahid Hoseinzade",
    author_email="javad.ebadi.1990@gmail.com, vahid.hoseinzade64@gmail.com",
    description="A simple python wrapper for inspirehep API",
    name="pyinspirehep",
    packages=find_packages(include=["pyinspirehep", "pyinspirehep.*"]),
    version="1.1.0",
    install_requires=['requests'],
    python_requires='>=3.7',
    license='MIT',
    url='https://github.com/javadebadi/pyinspirehep',
    long_description=long_description,
    long_description_content_type='text/markdown'
)
